// 2011 Hiroyuki Ogasawara (http://wlog.flatlib.jp)
// vim:ts=4 sw=4:

package jp.flatlib.ap02a;

import android.content.Context;
import android.content.res.Resources;
import android.renderscript.RSSurfaceView;
import android.renderscript.RenderScriptGL;
import android.renderscript.Float4;
import android.renderscript.Allocation;
import android.renderscript.ProgramVertex;
import android.renderscript.ProgramFragment;
import android.renderscript.ProgramStore;
import android.renderscript.ProgramRaster;
import android.renderscript.Mesh;
import android.view.SurfaceHolder;
import android.util.Log;
import java.io.InputStream;
import java.io.IOException;


public class SimpleView extends RSSurfaceView {

    private RenderScriptGL      mRS;
    private Resources           mRes;
    private Context             mContext;

	private float		mWidth;
	private float		mHeight;

	private	ScriptField_ConstantType2D	a_vconst2d;

    public SimpleView( Context context ) {
        super( context );
        createRS();
    }


	public static String LoadResource( Context context, int resid ) {
		InputStream	is= context.getResources().openRawResource( resid );
		byte[]	buf= null;
		try {
			int	size= is.available();
			buf= new byte[size];
			is.read( buf );
			is.close();
		}catch( IOException e ){
		}
		return	new String( buf );
	}


    private void createRS() {
        if( mRS != null ){
            return;
        }

        RenderScriptGL.SurfaceConfig    sc= new RenderScriptGL.SurfaceConfig();
        mRS= createRenderScriptGL( sc );

        mContext= getContext();
        mRes= mContext.getResources();
    }


    private void setupRS() {

		createRS();

        ScriptC_rendermesh  mScript= new ScriptC_rendermesh( mRS, mRes, R.raw.rendermesh );

		// Mesh
		Mesh.TriangleMeshBuilder	builder= new Mesh.TriangleMeshBuilder( mRS, 2, Mesh.TriangleMeshBuilder.COLOR );

		builder.setColor( 1f, 0f, 0f, 1f ).addVertex(  10.0f,  10.0f );
		builder.setColor( 0f, 1f, 0f, 1f ).addVertex( 280.0f,  10.0f );
		builder.setColor( 0f, 0f, 1f, 1f ).addVertex(  10.0f, 280.0f );
		builder.setColor( 1f, 1f, 1f, 1f ).addVertex( 280.0f, 280.0f );

		builder.addTriangle( 0, 2, 1 );
		builder.addTriangle( 1, 2, 3 );
		Mesh	mesh= builder.create( true );


		// Uniform
		a_vconst2d= new ScriptField_ConstantType2D( mRS, 1, Allocation.USAGE_GRAPHICS_CONSTANTS );
		//a_vconst2d.set_Transform( 0, new Float4( 2.0f/mWidth, -2.0f/mHeight, -1.0f, 1.0f ), true );
		mScript.bind_vconst( a_vconst2d );


		// VertexShader
		ProgramVertex.Builder	vsh_builder= new ProgramVertex.Builder( mRS );
		vsh_builder.setShader( LoadResource( mContext, R.raw.mesh2dc_vsh ) );
		vsh_builder.addConstant( a_vconst2d.getType() );
		vsh_builder.addInput( mesh.getVertexAllocation( 0 ).getType().getElement() );
		ProgramVertex	vsh= vsh_builder.create();
		vsh.bindConstants( a_vconst2d.getAllocation(), 0 );


		// PixelShader
		ProgramFragment.Builder	psh_builder= new ProgramFragment.Builder( mRS );
		psh_builder.setShader( LoadResource( mContext, R.raw.mesh2dc_psh ) );
		ProgramFragment	psh= psh_builder.create();

		// Global

		mScript.set_vsh( vsh );
		mScript.set_psh( psh );
		mScript.set_mesh( mesh );

        mRS.bindRootScript( mScript );
    }

    @Override
    public void onAttachedToWindow() {
        super.onAttachedToWindow();
        createRS();
    }

    @Override
    public void onDetachedFromWindow() {
        if( mRS != null ){
            mRS= null;
            destroyRenderScriptGL();
        }
    }

    @Override
    public void surfaceChanged( SurfaceHolder holder, int format, int w, int h ) {
		super.surfaceChanged( holder, format, w, h );
		mWidth= w;
		mHeight= h;
        setupRS();
    }
}



